// Definitions by: Peter Safranek <https://github.com/pe8ter>
declare function extend(obj1: object, ...objn: any[]): object;
declare function extend(deep: boolean, obj1: object, ...objn: any[]): object;
export default extend;
